//==============================================================================
// Unit Name     : ScrollText
// Author        : John David Hall
// Documention   : None
// Purpose       : To provide a simple scroll text component
// Date Created  : 26/02/1999
// Date Modified : 23/09/1999
//==============================================================================
unit Scrolltext;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Extctrls;

type

  TScrollType = (scNone, scLeft, scRight);

  TScrollMethod = (smChar, smPixel);

  TByteArray = Array [0..0] of byte;

  //============================================================================
  // PURPOSE : The base class from which the Scroll text component inherits from
  // AUTHOR  : John David Hall
  // DATE CREATED : 26/02/1999 Date Modified :   /  /
  //============================================================================
  TCustomScroll = class(TLabel)
  private
    { Private declarations }
    FMess : String;             //Text to scroll
    FCount : Integer;           //Total size of scroll message
    FSize : Integer;            //Scroll size area
    FPosition : Integer;        //Current scroll position
    FScrollType : TScrollType;
    FSpeed : word;
    FTimer  : TTimer;
    FOldCap : String;
    FScrollMethod : TScrollMethod;
    function GetScrollType : TScrollType;
    procedure SetScrollType (TY : TScrolltype);
    function GetScrollMethod : TScrollMethod;
    procedure SetScrollMethod (TM : TScrollMethod);
    function GetSpeed : Word;
    procedure SetSpeed (SP : Word);
    procedure SetMessage(AnMessage : string);
  protected
    { Protected declarations }
    Property ScrollMessage : String  read FMess write SetMessage;
    Property Scrolltype : TScrollType read GetScrolltype Write SetScrollType;
    Property Speed : Word read GetSpeed Write SetSpeed;
    Property ScrollMethod : TScrollMethod Read GetScrollMethod Write SetScrollMethod;

    procedure DoMoveMessage(Sender: TObject);
    procedure MoveCharMessage;
    procedure MovePixelMessage;
    procedure Recal;
    procedure Loaded; override;
  public
    { Public declarations }
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;

  Published
    { Published declarations }
  end;

  //============================================================================
  // PURPOSE : The component that is Registered
  // AUTHOR  : John David Hall
  // DATE CREATED : 26/02/1999 Date Modified :   /  /
  //============================================================================
  TScrollText = class(TCustomScroll)
  Published
    { Published declarations }
    Property ScrollMessage;
    Property Scrolltype;
    Property ScrollMethod;
    Property Speed;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('DarkAges', [TScrollText]);
end;

//==============================================================================
// PURPOSE : The default constructor for this class
// AUTHOR  : John David Hall
// DATE CREATED : 26/02/1999 Date Modified :   /  /
// INPUT PARAMS  : The Owner of the object
//==============================================================================
constructor TCustomScroll.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);

  //Set up the scoll timer
  FTimer := TTimer.Create(Self);
  FTimer.Interval := 50;
  FTimer.OnTimer := DoMoveMessage;
  FTimer.Enabled := False;

  //Set up the scroll text display
  Caption := FMess;
  AutoSize := False;
  WordWrap := False;
end;

//==============================================================================
// PURPOSE : The default destructor for this class
// AUTHOR  : John David Hall
// DATE CREATED : 26/02/1999 Date Modified :   /  /
//==============================================================================
destructor TCustomScroll.Destroy;
begin
 Caption := FOldCap;
 FTimer.Free;
 inherited Destroy;
end;

//==============================================================================
// PURPOSE : To return the type of scolling that this object is using
// AUTHOR  : John David Hall
// DATE CREATED : 26/02/1999 Date Modified :   /  /
// OUTPUT PARAMS : The type of scrolling
//==============================================================================
function TCustomScroll.GetScrollType : TScrollType;
begin
  result := FScrolltype;
end;

//==============================================================================
// PURPOSE : To set the type of scrolling that this object will use
// AUTHOR  : john David Hall
// DATE CREATED : 26/02/1999 Date Modified :   /  /
// INPUT PARAMS  : The Scroll Type E.G. scRight
//==============================================================================
procedure TCustomScroll.SetScrollType (TY : TScrollType);
begin
  FScrollType := TY;
  if TY = scNone then
    FTimer.Enabled := False
  else
    FTimer.Enabled := True;
end;

//==============================================================================
// PURPOSE : To return the method of scolling
// AUTHOR  : John David Hall
// DATE CREATED : 26/02/1999 Date Modified :   /  /
// OUTPUT PARAMS : The method this object uses for scolling
//==============================================================================
function TCustomScroll.GetScrollMethod : TScrollMethod;
begin
  Result := FScrollMethod;
end;

//==============================================================================
// PURPOSE : To set the method of scrolling for this object
// AUTHOR  : John David Hall
// DATE CREATED : 26/02/1999 Date Modified :   /  /
// INPUT PARAMS  : The method for scolling E.G. smPixel
//==============================================================================
procedure TCustomScroll.SetScrollMethod (TM : TScrollMethod);
begin
  FScrollMethod := TM;
end;

//==============================================================================
// PURPOSE : To set the message that the object will scroll
// AUTHOR  : john David Hall
// DATE CREATED : 26/02/1999 Date Modified :   /  /
// INPUT PARAMS : A string containg the message to be scrolled
//==============================================================================
procedure TCustomScroll.SetMessage(AnMessage : string);
begin
  if AnMessage <> '' then
    FMess := AnMessage;
end;//

//==============================================================================
// PURPOSE : To Set the speed at which the object will scroll the message
// AUTHOR  : John David Hall
// DATE CREATED : 26/02/1999 Date Modified :   /  /
// INPUT PARAMS  : The interval in of milliseconds before it scrolls the message
//==============================================================================
procedure TCustomScroll.SetSpeed (SP : Word);
begin
  if SP <> 0 then
    FSpeed := SP
  else
    FSpeed := 500;
  FTimer.Interval := FSpeed;
end;

//==============================================================================
// PURPOSE : To return the speed at which the object scrolls the message
// AUTHOR  : John David Hall
// DATE CREATED : 26/02/1999 Date Modified :   /  /
// OUTPUT PARAMS : The interval in milliseconds E.G. 500 (half a second)
//==============================================================================
function TCustomScroll.GetSpeed : Word;
begin
  Result := FSpeed;
end;

procedure TCustomScroll.Loaded;
begin
  inherited Loaded;
  FOldCap := Self.Caption;
  if FScrollMethod = SMPixel then
    Caption := '';
  Recal;
end;

//==============================================================================
// PURPOSE : To work out the position from which to display the scroll message
// AUTHOR  : John David Hall
// DATE CREATED : 26/02/1999 Date Modified :   /  /
//==============================================================================
procedure TCustomScroll.Recal;
begin
  FCount := Length (FMess);
  FSize := Round(Width / Font.Size);  //works to how many characters can be displayed at once
  if FScrollMethod = SMChar then
  begin
    if FScrolltype = SCRight then
      Fposition := Fsize
    else
      Fposition := -Fcount;
  end
  else
  begin
    if FScrolltype = SCRight then
      Fposition := Width
    else
      Fposition := -Canvas.TextWidth(FMess);
  end;
end;

//==============================================================================
// PURPOSE : To update the message in the scroll text object
// AUTHOR  : John David Hall
// DATE CREATED : 26/02/1999 Date Modified :   /  /
//==============================================================================
procedure TCustomScroll.DoMoveMessage(Sender : TObject);
begin
  if not(csDesigning in ComponentState) then
  begin
    if FScrollMethod = smChar then
      MoveCharMessage
    else
      MovePixelMessage;
  end;
end;

//==============================================================================
// PURPOSE : To update teh scroll text message using the pixel method
// AUTHOR  : John David Hall
// DATE CREATED : 26/02/1999 Date Modified :   /  /
//==============================================================================
procedure TCustomScroll.MovePixelMessage;
begin
  if FScrollType = SCRight then
  begin
    Dec (FPosition,4);
    if FPosition + Canvas.TextWidth(FMess) < 0 then
      Fposition := Width;
  end
  else
  begin
    inc (FPosition,4);
    if FPosition > Canvas.TextWidth(FMess) then
      FPosition := -Canvas.TextWidth(FMess);
  end;
  Canvas.Lock;
  Refresh;
  Canvas.TextOut(FPosition, 0, FMess);
  Canvas.Unlock;
end;

//==============================================================================
// PURPOSE : To update teh scroll text message using the character method
// AUTHOR  : John David Hall
// DATE CREATED : 26/02/1999 Date Modified :   /  /
//==============================================================================
procedure TCustomScroll.MoveCharMessage;
var Display, Whitespace : string;
    Loop : integer;
begin
  if FScrollType = SCRight then
  begin
    Dec (Fposition);
    if Fposition < -Fcount then
      Fposition := FSize
  end
  else
  begin
    Inc (FPosition);
    if FPosition > FSize then
      FPosition := -FCount;
  end;

  if FPosition <= 0 then
  begin
    Display := '';
    Display := copy (FMess, -Fposition, FCount + FPosition+1);
  end
  else
  begin
    whitespace := '';
    repeat
      whitespace := whitespace + ' ';
    until Canvas.TextWidth(WhiteSpace) >= Round(FPosition * Font.Size);
    Display := WhiteSpace;
    Loop := 1;
    repeat
      Display := Display + FMess[Loop];
      Inc(Loop)
    until (Canvas.TextWidth(Display) >= Round(FSize * Font.Size)) or
          (Loop > FCount);
  end;
  Caption := Display;
end;

end.
